package com.tzld.piaoquan.featurestools.util;

import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
public class DeepSeekAPI {

    /**
     * 聊天端点
     */
    static String chatEndpoint = "https://api.deepseek.com/chat/completions";
    /**
     * api密匙
     */
    static String apiKey = "Bearer sk-cfd2df92c8864ab999d66a615ee812c5";

    private static final String systemPrompt = "请基于广告视频分析报告进行结构化关键词提取，按照以下要求输出JSON：\n" +
            "\n" +
            "1. 仅分为钩子、价值、逼单三个模块\n" +
            "2. 每个模块直接输出关键词数组\n" +
            "3. 合并专属与泛化词汇不做区分\n" +
            "4. 只保留和广告相关的关键性词汇\n" +
            "5. 保留原始模块定义：\n" +
            "   钩子 - 吸引注意的元素\n" +
            "   价值 - 产品提供的核心利益\n" +
            "   逼单 - 促成转化的手段\n" +
            "\n" +
            "输出结构：\n" +
            "{\n" +
            "  \"hook\": [],\n" +
            "  \"value\": [],\n" +
            "  \"urgency\": []\n" +
            "}\n" +
            "\n" +
            "提取规则：\n" +
            "- 从文案/画面/行为中提炼核心概念\n" +
            "- 保留数字、特殊表述等关键信息\n" +
            "- 每个关键词不超过8个汉字\n" +
            "- 总条目数控制在15-20个";


    /**
     * 发送消息
     *
     * @param txt 内容
     * @return {@link String}
     */
    public static String chat(String txt) {
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("model", "deepseek-chat");
        paramMap.put("temperature", 0.3);
        Map<String, String> responseFormat = new HashMap<>();
        responseFormat.put("type", "json_object");
        paramMap.put("response_format", responseFormat);
        List<Map<String, String>> dataList = new ArrayList<>();
        dataList.add(new HashMap<String, String>() {{
            put("role", "system");
            put("content", systemPrompt);
        }});
        dataList.add(new HashMap<String, String>() {{
            put("role", "user");
            put("content", txt);
        }});
        paramMap.put("messages", dataList);
        JSONObject message = null;
        try {
            String body = HttpRequest.post(chatEndpoint)
                    .header("Authorization", apiKey)
                    .header("Content-Type", "application/json")
                    .body(JSONObject.toJSONString(paramMap))
                    .timeout(200000)
                    .execute()
                    .body();
            JSONObject jsonObject = JSONObject.parseObject(body);
            JSONArray choices = jsonObject.getJSONArray("choices");
            JSONObject result = choices.getJSONObject(0);
            message = result.getJSONObject("message");
            return message.getString("content");
        } catch (Exception e) {
            log.error("deepseek chat error", e);
        }
        return "";
    }

    public static void main(String[] args) {
        String txt = "好的，下面是对您上传的广告视频的分析报告。\n" +
                "\n" +
                "广告视频分析报告\n" +
                "\n" +
                "一、钩子 (吸引观众注意力的部分)\n" +
                "\n" +
                "   文案：“人呐，一旦开始练习站桩，身体会发生哪些变化呢？”\n" +
                "   画面：视频一开始呈现两位中年人在绿地上练习站桩的画面，一位穿蓝色中式服装的男士在指导一位穿红色中式服装的女士。画面色彩鲜明，背景是绿色的草地和带有倒影的窗户，营造出一种轻松自然的感觉。\n" +
                "   吸引注意的方式：提出问题。视频通过提问“人呐，一旦开始练习站桩，身体会发生哪些变化呢？”直接引发观众的好奇心，尤其是对于有健康养生需求的40岁以上人群，更能激起他们了解站桩益处的兴趣。\n" +
                "\n" +
                "二、价值 (视频传递的产品/服务信息)\n" +
                "\n" +
                "   产品/服务：视频宣传的是“站桩入门课”，由千尺学堂提供，适合40岁以上人群。\n" +
                "   核心卖点：站桩的核心在于通过内部运动带动外部，吸收周围能量，将散掉的精气重新收回体内，从而达到身体轻松，排出湿气，心情舒畅的效果。\n" +
                "   解决痛点：视频旨在解决中老年人群普遍存在的身体疲惫、湿气重、心情烦闷等问题。通过站桩练习，可以改善身体状况，提升精神状态。\n" +
                "\n" +
                "三、逼单 (引导用户下单的部分)\n" +
                "\n" +
                "   营销话术：“现在报名5天的课程，一分钱不用花。”\n" +
                "   行动引导：视频通过画面上的红色箭头和文案提示，引导用户点击视频下方的链接来领取课程。这种引导方式简洁直接，易于操作。\n" +
                "   紧迫感营造：视频没有明显的紧迫感营造手段，但是免费课程本身具有一定的吸引力，可能会促使感兴趣的观众尽快点击链接了解详情。";

        System.out.println(chat(txt));
    }

}
